@@ -75,12 +75,18 @@ fill_sample (GtkHTMLEditBodyProperties *d)
 }
 
 static void
-color_changed (GtkWidget *w, GdkColor *color, GtkHTMLEditBodyProperties *data)
+color_changed (GtkWidget *w, GdkColor *color, gboolean by_user, GtkHTMLEditBodyProperties *data)
 {
 	gint idx;
 
+	/* If the color was changed programatically there's not need to set things */
+	if (!by_user)
+		return;
+		
 	idx = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (w), "type"));
-	data->color [idx] = *color;
+	data->color [idx] = color
+		? *color
+		: html_colorset_get_color (data->cd->html->engine->defaultSettings->color_set, idx)->color;
 	data->color_changed [idx] = TRUE;
 	gtk_html_edit_properties_dialog_change (data->cd->properties_dialog);
 	fill_sample (data);
@@ -116,6 +122,7 @@ body_properties (GtkHTMLControlData *cd, gpointer *set_data)
 	GtkHTMLEditBodyProperties *data = g_new0 (GtkHTMLEditBodyProperties, 1);
 	GtkWidget *hbox, *vbox, *frame, *combo, *table;
 	GSList *group;
+	HTMLColor *color;
 
 	*set_data = data;
 	data->cd = cd;
@@ -157,25 +164,30 @@ body_properties (GtkHTMLControlData *cd, gpointer *set_data)
 	group = NULL;
 #define ADD_COLOR(x, ct, g) \
         data->color [ct] = html_colorset_get_color_allocated (cd->html->engine->painter, ct)->color; \
+        color = html_colorset_get_color (cd->html->engine->defaultSettings->color_set, ct); \
+        html_color_alloc (color, cd->html->engine->painter); \
 	combo = color_combo_new (NULL, _("Automatic"), \
-				 &data->color [ct], \
-				 "body_" g); \
+				 &color->color, \
+				 color_group_fetch ("body_" g, cd)); \
+        color_combo_set_color (COLOR_COMBO (combo), &data->color [ct]); \
         gtk_object_set_data (GTK_OBJECT (combo), "type", GINT_TO_POINTER (ct)); \
         gtk_signal_connect (GTK_OBJECT (combo), "changed", GTK_SIGNAL_FUNC (color_changed), data); \
 	hbox = gtk_hbox_new (FALSE, 3); \
 	gtk_box_pack_start (GTK_BOX (hbox), combo, FALSE, FALSE, 0); \
-	gtk_box_pack_start (GTK_BOX (hbox), gtk_label_new (_(x)), FALSE, FALSE, 0); \
+	gtk_box_pack_start (GTK_BOX (hbox), gtk_label_new (x), FALSE, FALSE, 0); \
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
-	ADD_COLOR ("Text", HTMLTextColor, "text");
-	ADD_COLOR ("Link", HTMLLinkColor, "link");
-	ADD_COLOR ("Background", HTMLBgColor, "bg");
+	ADD_COLOR (_("Text"), HTMLTextColor, "text");
+	ADD_COLOR (_("Link"), HTMLLinkColor, "link");
+	ADD_COLOR (_("Background"), HTMLBgColor, "bg");
 
 	gtk_container_add (GTK_CONTAINER (frame), vbox);
 	gtk_table_attach_defaults (GTK_TABLE (table), frame, 1, 2, 0, 1);
 	fill_sample (data);
 	/* gtk_idle_add (hide_preview, data); */
 
+	gtk_widget_show_all (table);
+
 	return table;
 }
 
