@@ -32,8 +32,13 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 
+#include <glade/glade.h>
 #include <gal/widgets/e-scroll-frame.h>
 
+#ifdef GNOME_GTKHTML_EDITOR_SHLIB
+#include <bonobo/bonobo-shlib-factory.h>
+#endif
+
 #include "Editor.h"
 
 #include "gtkhtml.h"
@@ -48,7 +53,6 @@
 #include "htmlpainter.h"
 #include "htmlplainpainter.h"
 
-
 #include "engine.h"
 #include "menubar.h"
 #include "persist-file-impl.h"
@@ -63,15 +67,20 @@
 #include "spell.h"
 #include "resolver-progressive-impl.h"
 
-#include "editor-control-factory.h"
 #include "gtkhtmldebug.h"
 
 #ifdef USING_OAF
-#define CONTROL_FACTORY_ID "OAFIID:GNOME_GtkHTML_EditorFactory"
+#define CONTROL_FACTORY_ID "OAFIID:GNOME_GtkHTML_Editor_Factory"
 #else
 #define CONTROL_FACTORY_ID "control-factory:html-editor"
 #endif
 
+#ifndef GNOME_GTKHTML_EDITOR_SHLIB
+static BonoboGenericFactory *factory;
+static gint active_controls = 0;
+
+#endif
+
 
 /* This is the initialization that can only be performed after the
    control has been embedded (signal "set_frame").  */
@@ -84,9 +93,6 @@ typedef struct _SetFrameData SetFrameData;
 
 GtkHTMLEditorAPI *editor_api;
 
-static BonoboGenericFactory *factory = NULL;
-static gint active_controls = 0;
-
 static void
 set_frame_cb (BonoboControl *control,
 	      gpointer data)
@@ -140,6 +146,7 @@ release (GtkWidget *widget, GdkEventButton *event, GtkHTMLControlData *cd)
 			run_dialog = TRUE;
 			break;
 		default:
+			;
 		}
 		if (run_dialog) {
 			cd->properties_dialog = gtk_html_edit_properties_dialog_new (cd, FALSE, _("Properties"));
@@ -180,6 +187,7 @@ release (GtkWidget *widget, GdkEventButton *event, GtkHTMLControlData *cd)
 				start = GTK_HTML_EDIT_PROPERTY_RULE;
 				break;
 			default:
+				;
 			}
 			gtk_html_edit_properties_dialog_add_entry (cd->properties_dialog,
 								   GTK_HTML_EDIT_PROPERTY_PARAGRAPH, _("Paragraph"),
@@ -330,6 +338,7 @@ html_button_pressed (GtkWidget *html, GdkEventButton *event, GtkHTMLControlData
 			gtk_signal_emit_stop_by_name (GTK_OBJECT (html), "button_press_event");
 		break;
 	default:
+		;
 	}
 
 	return FALSE;
@@ -339,6 +348,8 @@ static void
 destroy_control_data_cb (GtkObject *control, GtkHTMLControlData *cd)
 {
 	gtk_html_control_data_destroy (cd);
+
+#ifndef GNOME_GTKHTML_EDITOR_SHLIB
 	active_controls --;
 
 	if (active_controls)
@@ -346,6 +357,7 @@ destroy_control_data_cb (GtkObject *control, GtkHTMLControlData *cd)
 
 	bonobo_object_unref (BONOBO_OBJECT (factory));
 	gtk_main_quit ();
+#endif
 }
 
 static void
@@ -404,10 +416,8 @@ editor_set_format (GtkHTMLControlData *cd, gboolean format_html)
 		if (html->engine->window)
 			html_gdk_painter_realize (p, html->engine->window);
 
-		html_engine_set_painter (html->engine, HTML_PAINTER (p), 
-					 html->engine->width);
-		
-		html_engine_draw (html->engine, 0, 0, html->engine->width, html->engine->height);
+		html_engine_set_painter (html->engine, HTML_PAINTER (p));
+		html_engine_schedule_redraw (html->engine);
 	}
 		
 }
@@ -458,42 +468,37 @@ editor_set_prop (BonoboPropertyBag *bag,
 static void
 editor_control_construct (BonoboControl *control, GtkWidget *vbox)
 {
-	GtkHTMLControlData *control_data;
-	BonoboPersistStream *stream_impl;
-	BonoboPersistFile *file_impl;
+	GtkHTMLControlData *cd;
 	GtkWidget *html_widget;
 	BonoboPropertyBag  *pb;
 	BonoboArg          *def;
 
+#ifndef GNOME_GTKHTML_EDITOR_SHLIB
 	active_controls++;
-
+#endif
 	/* GtkHTML widget */
-
 	html_widget = gtk_html_new ();
 	gtk_html_load_empty (GTK_HTML (html_widget));
 	gtk_html_set_editable (GTK_HTML (html_widget), TRUE);
 
-	control_data = gtk_html_control_data_new (GTK_HTML (html_widget), vbox);
-	gtk_html_set_editor_api (GTK_HTML (html_widget), editor_api, control_data);
+	cd = gtk_html_control_data_new (GTK_HTML (html_widget), vbox);
+	gtk_html_set_editor_api (GTK_HTML (html_widget), editor_api, cd);
 
 	/* HTMLEditor::Engine */
-
-	control_data->editor_bonobo_engine = editor_engine_new (GTK_HTML (html_widget));
+	cd->editor_bonobo_engine = editor_engine_new (GTK_HTML (html_widget));
 	bonobo_object_add_interface (BONOBO_OBJECT (control), 
-				     BONOBO_OBJECT (control_data->editor_bonobo_engine));
+				     BONOBO_OBJECT (cd->editor_bonobo_engine));
 
 	/* Bonobo::PersistStream */
-
-	stream_impl = persist_stream_impl_new (GTK_HTML (html_widget));
-	bonobo_object_add_interface (BONOBO_OBJECT (control), BONOBO_OBJECT (stream_impl));
+	cd->persist_stream = persist_stream_impl_new (GTK_HTML (html_widget));
+	bonobo_object_add_interface (BONOBO_OBJECT (control), BONOBO_OBJECT (cd->persist_stream));
 
 	/* Bonobo::PersistFile */
-
-	file_impl = persist_file_impl_new (GTK_HTML (html_widget));
-	bonobo_object_add_interface (BONOBO_OBJECT (control), BONOBO_OBJECT (file_impl));
+	cd->persist_file = persist_file_impl_new (GTK_HTML (html_widget));
+	bonobo_object_add_interface (BONOBO_OBJECT (control), BONOBO_OBJECT (cd->persist_file));
 
 	/* PropertyBag */
-	pb = bonobo_property_bag_new (editor_get_prop, editor_set_prop, control_data);
+	pb = bonobo_property_bag_new (editor_get_prop, editor_set_prop, cd);
 	bonobo_control_set_properties (control, pb);
 
 	def = bonobo_arg_new (BONOBO_ARG_BOOLEAN);
@@ -522,39 +527,18 @@ editor_control_construct (BonoboControl *control, GtkWidget *vbox)
 	   handle that.  */
 
 	gtk_signal_connect (GTK_OBJECT (control), "set_frame",
-			    GTK_SIGNAL_FUNC (set_frame_cb), control_data);
+			    GTK_SIGNAL_FUNC (set_frame_cb), cd);
 
 	gtk_signal_connect (GTK_OBJECT (control), "destroy",
-			    GTK_SIGNAL_FUNC (destroy_control_data_cb), control_data);
+			    GTK_SIGNAL_FUNC (destroy_control_data_cb), cd);
 
 	gtk_signal_connect (GTK_OBJECT (html_widget), "url_requested",
 			    GTK_SIGNAL_FUNC (url_requested_cb), control);
 
 	gtk_signal_connect (GTK_OBJECT (html_widget), "button_press_event",
-			    GTK_SIGNAL_FUNC (html_button_pressed), control_data);
+			    GTK_SIGNAL_FUNC (html_button_pressed), cd);
 
-	control_data->control = control;
-}
-
-static BonoboObject *
-editor_control_factory (BonoboGenericFactory *factory,
-			gpointer closure)
-{
-	BonoboControl *control;
-	GtkWidget *vbox;
-
-	vbox = gtk_vbox_new (FALSE, 0);
-	gtk_widget_show (vbox);
-
-	/* g_warning ("Creating a new GtkHTML editor control."); */
-	control = bonobo_control_new (vbox);
-
-	if (control)
-		editor_control_construct (control, vbox);
-	else
-		gtk_widget_unref (vbox);
-
-	return BONOBO_OBJECT (control);
+	cd->control = control;
 }
 
 static gboolean
@@ -677,18 +661,120 @@ new_editor_api ()
 	editor_api->create_input_line  = editor_api_create_input_line;
 }
 
-void
-editor_control_factory_init (void)
+static void
+editor_control_init (void)
 {
-	if (factory != NULL)
-		return;
-
 	new_editor_api ();
 	gdk_rgb_init ();
+	glade_gnome_init ();
+}
+
+static gboolean editor_control_initialized = FALSE;
+
+static BonoboObject *
+editor_control_factory (BonoboGenericFactory *factory, gpointer closure)
+{
+	BonoboControl *control;
+	GtkWidget *vbox;
+
+	if (!editor_control_initialized)
+		editor_control_init ();
+
+	vbox = gtk_vbox_new (FALSE, 0);
+	gtk_widget_show (vbox);
+
+	/* g_warning ("Creating a new GtkHTML editor control."); */
+	control = bonobo_control_new (vbox);
+
+	if (control){
+		editor_control_construct (control, vbox);
+		return BONOBO_OBJECT (control);
+	} else {
+		gtk_widget_unref (vbox);
+		return NULL;
+	}
+}
+
+#ifdef GNOME_GTKHTML_EDITOR_SHLIB
+
+BONOBO_OAF_SHLIB_FACTORY ("OAFIID:GNOME_GtkHTML_Editor_Factory", "GNOME HTML Editor factory", editor_control_factory, NULL);
+
+#else
+
+#ifdef USING_OAF
+
+#include <liboaf/liboaf.h>
+
+static CORBA_ORB
+init_corba (int *argc, char **argv)
+{
+	gnome_init_with_popt_table ("html-editor-factory", "0.0",
+				    *argc, argv, oaf_popt_options, 0, NULL);
+
+	return oaf_init (*argc, argv);
+}
+
+#else
+
+#include <libgnorba/gnorba.h>
+
+static CORBA_ORB
+init_corba (int *argc, char **argv)
+{
+	CORBA_Environment ev;
+
+	CORBA_exception_init (&ev);
+
+	gnome_CORBA_init_with_popt_table ("html-editor-factory", "0.0",
+					  argc, argv,
+					  NULL, 0, NULL,
+					  GNORBA_INIT_SERVER_FUNC,
+					  &ev);
+	if (ev._major != CORBA_NO_EXCEPTION)
+		g_error (_("Could not initialize GNORBA"));
+
+	CORBA_exception_free (&ev);
+
+	return CORBA_OBJECT_NIL;
+}
+
+#endif
+
+static void
+init_bonobo (int *argc, char **argv)
+{
+	if (bonobo_init (init_corba (argc, argv), CORBA_OBJECT_NIL, CORBA_OBJECT_NIL) == FALSE)
+		g_error (_("Could not initialize Bonobo"));
+}
+
+int
+main (int argc, char **argv)
+{
+#ifdef GTKHTML_HAVE_GCONF
+	GError  *gconf_error  = NULL;
+#endif
+
+	/* Initialize the i18n support */
+	bindtextdomain(PACKAGE, GNOMELOCALEDIR);
+	textdomain(PACKAGE);
+
+	init_bonobo (&argc, argv);
+#ifdef GTKHTML_HAVE_GCONF
+	if (!gconf_init (argc, argv, &gconf_error)) {
+		g_assert (gconf_error != NULL);
+		g_error ("GConf init failed:\n  %s", gconf_error->message);
+		return 1;
+	}
+#endif
+
 	factory = bonobo_generic_factory_new (CONTROL_FACTORY_ID,
 					      editor_control_factory,
 					      NULL);
-
 	if (factory == NULL)
 		g_error ("I could not register the GNOME_GtkHTML_Editor factory.");
+
+	bonobo_main ();
+
+	return 0;
 }
+#endif /* GNOME_GTKHTML_EDITOR_SHLIB */
