@@ -31,7 +31,6 @@
 
 #include "config.h"
 #include "properties.h"
-#include "dialog.h"
 #include "rule.h"
 #include "utils.h"
 
@@ -286,6 +285,8 @@ rule_widget (GtkHTMLEditRuleProperties *data)
 
 	data->disable_change = FALSE;
 
+	gtk_widget_show_all (vbox);
+
 	return vbox;
 }
 
