@@ -225,7 +225,6 @@ update_styles (GtkHTML *html)
 {
 	GtkHTMLParagraphStyle paragraph_style;
 	GtkHTMLParagraphAlignment alignment;
-	HTMLClueFlowStyle clueflow_style;
 	HTMLEngine *engine;
 	guint indentation;
 
@@ -234,33 +233,31 @@ update_styles (GtkHTML *html)
 	if (! html_engine_get_editable (html->engine))
 		return;
 
-	engine = html->engine;
-
-	clueflow_style = html_engine_get_current_clueflow_style (engine);
-	paragraph_style = clueflow_style_to_paragraph_style (clueflow_style);
+	engine          = html->engine;
+	paragraph_style = clueflow_style_to_paragraph_style (html_engine_get_current_clueflow_style (engine));
 
 	if (paragraph_style != html->priv->paragraph_style) {
 		html->priv->paragraph_style = paragraph_style;
-		gtk_signal_emit (GTK_OBJECT (html), signals[CURRENT_PARAGRAPH_STYLE_CHANGED],
+		gtk_signal_emit (GTK_OBJECT (html), signals [CURRENT_PARAGRAPH_STYLE_CHANGED],
 				 paragraph_style);
 	}
 
 	indentation = html_engine_get_current_clueflow_indentation (engine);
 	if (indentation != html->priv->paragraph_indentation) {
-		html->priv->paragraph_style = paragraph_style;
-		gtk_signal_emit (GTK_OBJECT (html), signals[CURRENT_PARAGRAPH_STYLE_CHANGED], paragraph_style);
+		html->priv->paragraph_indentation = indentation;
+		gtk_signal_emit (GTK_OBJECT (html), signals [CURRENT_PARAGRAPH_INDENTATION_CHANGED], indentation);
 	}
 
 	alignment = html_alignment_to_paragraph (html_engine_get_current_clueflow_alignment (engine));
  	if (alignment != html->priv->paragraph_alignment) {
 		html->priv->paragraph_alignment = alignment;
-		gtk_signal_emit (GTK_OBJECT (html), signals[CURRENT_PARAGRAPH_ALIGNMENT_CHANGED], alignment);
+		gtk_signal_emit (GTK_OBJECT (html), signals [CURRENT_PARAGRAPH_ALIGNMENT_CHANGED], alignment);
 	}
 
 	if (html_engine_update_insertion_font_style (engine))
-		gtk_signal_emit (GTK_OBJECT (html), signals[INSERTION_FONT_STYLE_CHANGED], engine->insertion_font_style);
+		gtk_signal_emit (GTK_OBJECT (html), signals [INSERTION_FONT_STYLE_CHANGED], engine->insertion_font_style);
 	if (html_engine_update_insertion_color (engine))
-		gtk_signal_emit (GTK_OBJECT (html), signals[INSERTION_COLOR_CHANGED], engine->insertion_color);
+		gtk_signal_emit (GTK_OBJECT (html), signals [INSERTION_COLOR_CHANGED], engine->insertion_color);
 
 	/* TODO add insertion_url_or_targed_changed signal */
 	html_engine_update_insertion_url_and_target (engine);
@@ -633,7 +630,6 @@ key_press_event (GtkWidget *widget,
 {
 	GtkHTML *html = GTK_HTML (widget);
 	gboolean retval, update = TRUE;
-	gint position = html->engine->cursor->position;
 
 	html->binding_handled = FALSE;
 	html->priv->update_styles = FALSE;
@@ -666,10 +662,8 @@ key_press_event (GtkWidget *widget,
 	if (retval && html_engine_get_editable (html->engine))
 		html_engine_reset_blinking_cursor (html->engine);
 
-	if (retval) {
-		if (position != html->engine->cursor->position && update)
-			update_styles (html);
-	}
+	if (retval && update)
+		update_styles (html);
 
 	/* printf ("retval: %d\n", retval); */
 
@@ -1021,7 +1015,7 @@ button_press_event (GtkWidget *widget,
 						     x + engine->x_offset,
 						     y + engine->y_offset);
 				update_styles (html);
-				gtk_html_request_paste (html, 0, event->time);
+				gtk_html_request_paste (html, GDK_SELECTION_PRIMARY, 0, event->time);
 				return TRUE;
 			}
 			break;
@@ -1063,10 +1057,14 @@ button_press_event (GtkWidget *widget,
 			break;
 		}
 	} else if (event->button == 1 && html->allow_selection) {
-		if (event->type == GDK_2BUTTON_PRESS)
+		if (event->type == GDK_2BUTTON_PRESS) {
 			gtk_html_select_word (html);
-		else if (event->type == GDK_3BUTTON_PRESS)
+			html->in_selection = TRUE;
+		}
+		else if (event->type == GDK_3BUTTON_PRESS) {
 			gtk_html_select_line (html);
+			html->in_selection = TRUE;
+		}
 	}
 
 	return TRUE;
@@ -1088,12 +1086,16 @@ button_release_event (GtkWidget *widget,
 		html->button1_pressed = FALSE;
 		
 		if (html->pointer_url != NULL && ! html->in_selection)
-			gtk_signal_emit (GTK_OBJECT (widget), signals[LINK_CLICKED], html->pointer_url);
+			gtk_signal_emit (GTK_OBJECT (widget), 
+					 signals[LINK_CLICKED], 
+					 html->pointer_url);
 	}
 
 	if (html->in_selection) {
-		if (html_engine_get_editable (html->engine))
-			gtk_html_copy (html);
+		/* Copy to primary save area */
+		html_engine_copy_object(html->engine,
+					&html->priv->primary,
+					&html->priv->primary_len);
 		html->in_selection = FALSE;
 		update_styles (html);
 	}
@@ -1204,9 +1206,22 @@ selection_get (GtkWidget        *widget,
 	g_return_if_fail (GTK_IS_HTML (widget));
 	
 	html = GTK_HTML (widget);
-	selection_string = html->engine->clipboard
-		? html_object_get_selection_string (html->engine->clipboard)
-		: html_engine_get_selection_string (html->engine);
+	if (selection_data->selection == GDK_SELECTION_PRIMARY)
+	  {
+		if (html->priv->primary) {
+			selection_string =
+			   html_object_get_selection_string (html->priv->primary);
+			g_print("primary paste: `%s'\n", selection_string);
+		}
+	  }
+	else	/* CLIPBOARD */
+	  {
+		if (html->engine->clipboard) {
+			selection_string =
+			   html_object_get_selection_string (html->engine->clipboard);
+			g_print("clipboard paste: `%s'\n", selection_string);
+		}
+	  }
 
 	/*
 	 * FIXME we should make e_utf8_to/from_string_target and 
@@ -1280,10 +1295,38 @@ selection_received (GtkWidget *widget,
 	
 	printf("got selection from system\n");
 
+	/* If the Widget is editable,
+	** and we are the owner of the atom requested
+	** then we are pasting between ourself and we
+	** need not do all the conversion.
+	*/
 	if (html_engine_get_editable (GTK_HTML (widget)->engine)
-	    && widget->window == gdk_selection_owner_get (selection_data->selection) && GTK_HTML (widget)->engine->clipboard) {
-		html_engine_paste (GTK_HTML (widget)->engine);
-		return;
+	    && widget->window == gdk_selection_owner_get (selection_data->selection)) {
+
+		/* Check which atom was requested (PRIMARY or CLIPBOARD) */
+		if (selection_data->selection == gdk_atom_intern ("CLIPBOARD", FALSE)
+ 			&& GTK_HTML (widget)->engine->clipboard) {
+
+			html_engine_paste (GTK_HTML (widget)->engine);
+			return;
+
+		} else if (selection_data->selection == GDK_SELECTION_PRIMARY
+ 			&& GTK_HTML (widget)->priv->primary) {
+
+			HTMLObject *copy;
+			guint len = 0;
+
+			/* Take a copy so that it can be put in the undo
+			** buffer
+			*/
+			copy = html_object_op_copy (GTK_HTML (widget)->priv->primary,
+				 GTK_HTML (widget)->engine, NULL, NULL, &len);
+
+			html_engine_paste_object (GTK_HTML (widget)->engine,
+				copy,
+				GTK_HTML (widget)->priv->primary_len);
+			return;
+		}
 	}
 
 	/* **** IMPORTANT **** Check to see if retrieval succeeded  */
@@ -1293,7 +1336,7 @@ selection_received (GtkWidget *widget,
 		gint type = GTK_HTML (widget)->priv->last_selection_type;
 		
 		/* now, try again with next selection type */
-		if (!gtk_html_request_paste (GTK_HTML (widget), type + 1, time))
+		if (!gtk_html_request_paste (GTK_HTML (widget), selection_data->selection, type + 1, time))
 			g_warning ("Selection retrieval failed\n");
 		return;
 	}
@@ -1332,7 +1375,7 @@ selection_received (GtkWidget *widget,
 }  
 
 gint
-gtk_html_request_paste (GtkHTML *html, gint type, gint32 time)
+gtk_html_request_paste (GtkHTML *html, GdkAtom selection, gint type, gint32 time)
 {
 	GdkAtom format_atom;
 	static char *formats[] = {"UTF8_STRING", "UTF-8", "STRING"};
@@ -1351,8 +1394,8 @@ gtk_html_request_paste (GtkHTML *html, gint type, gint32 time)
 	if (format_atom == GDK_NONE) {
 		g_warning("Could not get requested atom\n");
 	}
-	/* And request the format target for the primary selection */
-	gtk_selection_convert (GTK_WIDGET (html), GDK_SELECTION_PRIMARY, format_atom,
+	/* And request the format target for the required selection */
+	gtk_selection_convert (GTK_WIDGET (html), selection, format_atom,
 			       time);
 	return TRUE;
 }
@@ -1880,12 +1923,17 @@ init (GtkHTML* html)
 	html->priv->paragraph_indentation = 0;
 	html->priv->insertion_font_style = GTK_HTML_FONT_STYLE_DEFAULT;
 	html->priv->last_selection_type = -1;
+	html->priv->primary = NULL;
+	html->priv->primary_len = 0;
 	html->priv->content_type = NULL;
 	html->priv->search_input_line = NULL;
 
 	gtk_selection_add_targets (GTK_WIDGET (html),
 				   GDK_SELECTION_PRIMARY,
 				   targets, n_targets);
+	gtk_selection_add_targets (GTK_WIDGET (html),
+				   gdk_atom_intern ("CLIPBOARD", FALSE),
+				   targets, n_targets);
 }
 
 
@@ -2407,7 +2455,7 @@ gtk_html_cut (GtkHTML *html)
 	g_return_if_fail (GTK_IS_HTML (html));
 
 	html_engine_cut (html->engine);
-	gtk_selection_owner_set (GTK_WIDGET (html), GDK_SELECTION_PRIMARY, GDK_CURRENT_TIME);
+	gtk_selection_owner_set (GTK_WIDGET (html), gdk_atom_intern ("CLIPBOARD", FALSE), GDK_CURRENT_TIME);
 }
 
 void
@@ -2417,7 +2465,7 @@ gtk_html_copy (GtkHTML *html)
 	g_return_if_fail (GTK_IS_HTML (html));
 
 	html_engine_copy (html->engine);
-	gtk_selection_owner_set (GTK_WIDGET (html), GDK_SELECTION_PRIMARY, GDK_CURRENT_TIME);
+	gtk_selection_owner_set (GTK_WIDGET (html), gdk_atom_intern ("CLIPBOARD", FALSE), GDK_CURRENT_TIME);
 }
 
 void
@@ -2426,7 +2474,7 @@ gtk_html_paste (GtkHTML *html)
 	g_return_if_fail (html != NULL);
 	g_return_if_fail (GTK_IS_HTML (html));
 
-	gtk_html_request_paste (html, 0, GDK_CURRENT_TIME);
+	gtk_html_request_paste (html, gdk_atom_intern ("CLIPBOARD", FALSE), 0, GDK_CURRENT_TIME);
 }
 
 
@@ -2743,45 +2791,23 @@ delete_one (HTMLEngine *e, gboolean forward)
 }
 
 inline static void
-inset_tab_or_indent_more_or_next_cell (GtkHTML *html)
+insert_tab_or_indent_more_or_next_cell (GtkHTML *html)
 {
 	HTMLEngine *e = html->engine;
-	HTMLTableCell *cell;
-
-	cell = html_engine_get_table_cell (e);
-	if (cell) {
-		html_engine_hide_cursor (e);
-		html_cursor_end_of_line (e->cursor, e);
-		html_cursor_forward (e->cursor, e);
-		if (HTML_IS_TABLE (e->cursor->object)) {
-			html_cursor_backward (e->cursor, e);
-			html_engine_insert_table_row (e, TRUE);
-		}
-		html_engine_show_cursor (e);
-		gtk_html_edit_make_cursor_visible (html);
-	} else if (!html_engine_is_selection_active (e)
-	    && html_clueflow_tabs (HTML_CLUEFLOW (e->cursor->object->parent), e->painter))
-		html_engine_insert_text (e, "\t", 1);
-	else
-		gtk_html_modify_indent_by_delta (html, +1);
+	if (!html_engine_next_cell (e, TRUE)) {
+		if (!html_engine_is_selection_active (e)
+		    && html_clueflow_tabs (HTML_CLUEFLOW (e->cursor->object->parent), e->painter))
+			html_engine_insert_text (e, "\t", 1);
+		else
+			gtk_html_modify_indent_by_delta (html, +1);
+	}
 }
 
 static void
 indent_less_or_prev_cell (GtkHTML *html)
 {
-	HTMLTableCell *cell;
-
-	cell = html_engine_get_table_cell (html->engine);
-	if (cell) {
-		HTMLEngine *e = html->engine;
-		html_engine_hide_cursor (e);
-		html_cursor_beginning_of_line (e->cursor, e);
-		html_cursor_backward (e->cursor, e);
-		html_engine_show_cursor (e);
-		gtk_html_edit_make_cursor_visible (html);
-	} else
+	if (!html_engine_prev_cell (html->engine))
 		gtk_html_modify_indent_by_delta (html, -1);
-
 }
 
 static gboolean
@@ -2829,22 +2855,25 @@ command (GtkHTML *html, GtkHTMLCommandType com_type)
 	/* editable commands only */
 	switch (com_type) {
 	case GTK_HTML_COMMAND_UNDO:
-		html_engine_undo (e);
+		gtk_html_undo (html);
 		break;
 	case GTK_HTML_COMMAND_REDO:
-		html_engine_redo (e);
+		gtk_html_redo (html);
 		break;
 	case GTK_HTML_COMMAND_COPY:
 		gtk_html_copy (html);
 		break;
 	case GTK_HTML_COMMAND_CUT:
 		gtk_html_cut (html);
+		html->priv->update_styles = TRUE;
 		break;
 	case GTK_HTML_COMMAND_CUT_LINE:
 		html_engine_cut_line (e);
+		html->priv->update_styles = TRUE;
 		break;
 	case GTK_HTML_COMMAND_PASTE:
 		gtk_html_paste (html);
+		html->priv->update_styles = TRUE;
 		break;
 	case GTK_HTML_COMMAND_INSERT_RULE:
 		html_engine_insert_rule (e, 0, 100, 2, TRUE, HTML_HALIGN_LEFT);
@@ -2852,6 +2881,7 @@ command (GtkHTML *html, GtkHTMLCommandType com_type)
 	case GTK_HTML_COMMAND_INSERT_PARAGRAPH:
 		html_engine_delete (e);
 		html_engine_insert_empty_paragraph (e);
+		html->priv->update_styles = TRUE;
 		break;
 	case GTK_HTML_COMMAND_DELETE:
 		if (e->mark != NULL
@@ -2859,12 +2889,14 @@ command (GtkHTML *html, GtkHTMLCommandType com_type)
 			html_engine_delete (e);
 		else
 			delete_one (e, TRUE);
+		html->priv->update_styles = TRUE;
 		break;
 	case GTK_HTML_COMMAND_DELETE_BACK:
 		if (html_engine_is_selection_active (e))
 			html_engine_delete (e);
 		else
 			delete_one (e, FALSE);
+		html->priv->update_styles = TRUE;
 		break;
 	case GTK_HTML_COMMAND_DELETE_BACK_OR_INDENT_DEC:
 		if (html_engine_is_selection_active (e))
@@ -2873,6 +2905,23 @@ command (GtkHTML *html, GtkHTMLCommandType com_type)
 			gtk_html_modify_indent_by_delta (html, -1);
 		else
 			delete_one (e, FALSE);
+		html->priv->update_styles = TRUE;
+		break;
+	case GTK_HTML_COMMAND_DELETE_TABLE:
+		html_engine_delete_table (e);
+		html->priv->update_styles = TRUE;
+		break;
+	case GTK_HTML_COMMAND_DELETE_TABLE_ROW:
+		html_engine_delete_table_row (e);
+		html->priv->update_styles = TRUE;
+		break;
+	case GTK_HTML_COMMAND_DELETE_TABLE_COLUMN:
+		html_engine_delete_table_column (e);
+		html->priv->update_styles = TRUE;
+		break;
+	case GTK_HTML_COMMAND_DELETE_TABLE_CELL_CONTENTS:
+		html_engine_delete_table_cell_contents (e);
+		html->priv->update_styles = TRUE;
 		break;
 	case GTK_HTML_COMMAND_SELECTION_MODE:
 		e->selection_mode = TRUE;
@@ -2973,7 +3022,7 @@ command (GtkHTML *html, GtkHTMLCommandType com_type)
 			gtk_html_modify_indent_by_delta (html, +1);
 		break;
 	case GTK_HTML_COMMAND_INSERT_TAB_OR_INDENT_MORE_OR_NEXT_CELL:
-		inset_tab_or_indent_more_or_next_cell (html);
+		insert_tab_or_indent_more_or_next_cell (html);
 		break;
 	case GTK_HTML_COMMAND_INDENT_DEC:
 		gtk_html_modify_indent_by_delta (html, -1);
@@ -3121,6 +3170,24 @@ command (GtkHTML *html, GtkHTMLCommandType com_type)
 	case GTK_HTML_COMMAND_TABLE_BORDER_WIDTH_ZERO:
 		html_engine_table_set_border_width (e, html_engine_get_table (e), 0, FALSE);
 		break;
+	case GTK_HTML_COMMAND_TABLE_SPACING_INC:
+		html_engine_table_set_spacing (e, html_engine_get_table (e), 1, TRUE);
+		break;
+	case GTK_HTML_COMMAND_TABLE_SPACING_DEC:
+		html_engine_table_set_spacing (e, html_engine_get_table (e), -1, TRUE);
+		break;
+	case GTK_HTML_COMMAND_TABLE_SPACING_ZERO:
+		html_engine_table_set_spacing (e, html_engine_get_table (e), 0, FALSE);
+		break;
+	case GTK_HTML_COMMAND_TABLE_PADDING_INC:
+		html_engine_table_set_padding (e, html_engine_get_table (e), 1, TRUE);
+		break;
+	case GTK_HTML_COMMAND_TABLE_PADDING_DEC:
+		html_engine_table_set_padding (e, html_engine_get_table (e), -1, TRUE);
+		break;
+	case GTK_HTML_COMMAND_TABLE_PADDING_ZERO:
+		html_engine_table_set_padding (e, html_engine_get_table (e), 0, FALSE);
+		break;
 	case GTK_HTML_COMMAND_TEXT_SET_DEFAULT_COLOR:
 		html_engine_set_color (e, NULL);
 		break;
@@ -3468,7 +3535,15 @@ gtk_html_command (GtkHTML *html, const gchar *command_name)
 	GtkEnumValue *val;
 
 	val = gtk_type_enum_find_value (GTK_TYPE_HTML_COMMAND, command_name);
-	return val ? command (html, val->value) : FALSE;
+	if (val) {
+		if (command (html, val->value)) {
+			if (html->priv->update_styles)
+				update_styles (html);
+			return TRUE;
+		}
+	}
+
+	return FALSE;
 }
 
 gboolean
