@@ -21,6 +21,10 @@
 
 /* Various debugging routines.  */
 
+#include <config.h>
+#include "gtkhtml-compat.h"
+
+#include <stdio.h>
 #include <stdarg.h>
 #include <string.h>
 
@@ -32,6 +36,8 @@
 #include "htmltablecell.h"
 #include "htmlclue.h"
 #include "htmlclueflow.h"
+#include "htmlframe.h"
+#include "htmlframeset.h"
 #include "htmliframe.h"
 #include "htmlengine.h"
 #include "htmltype.h"
@@ -185,6 +191,17 @@ gtk_html_debug_dump_object (HTMLObject *obj,
 		break;
 	case HTML_TYPE_IFRAME:
 		gtk_html_debug_dump_tree (GTK_HTML (HTML_IFRAME (obj)->html)->engine->clue, level + 1);
+		break;
+	case HTML_TYPE_FRAME:
+		gtk_html_debug_dump_tree (GTK_HTML (HTML_FRAME (obj)->html)->engine->clue, level + 1);
+		break;
+	case HTML_TYPE_FRAMESET: {
+		gint i;
+
+		for (i = 0; i < HTML_FRAMESET (obj)->frames->len; i++)
+			gtk_html_debug_dump_tree (g_ptr_array_index (HTML_FRAMESET (obj)->frames, i), level + 1);
+		}
+		break;
 	default:
 		break;
 	}
@@ -218,6 +235,17 @@ debug_word_width (HTMLText *t, gint level)
 	printf ("\n");
 }
 
+static void
+dump_data (GQuark key_id, gpointer data, gpointer user_data)
+{
+	gint i, level = GPOINTER_TO_INT (user_data);
+
+	for (i = 0; i < level; i++)
+		g_print ("\t");
+
+	printf ("%s: '%s'\n", g_quark_to_string (key_id), (gchar *) data);
+}
+
 static void
 dump_object_simple (HTMLObject *obj,
 		    gint level)
@@ -250,6 +278,9 @@ dump_object_simple (HTMLObject *obj,
 			 HTML_TABLE (obj)->totalRows, HTML_TABLE (obj)->totalCols);
 	} else
 		g_print ("%s\n", html_type_name (HTML_OBJECT_TYPE (obj)));
+
+	if (obj->object_data)
+		g_datalist_foreach (&obj->object_data, dump_data, GINT_TO_POINTER (level));
 }
 
 void
