@@ -140,22 +140,18 @@ html_engine_print_with_header_footer (HTMLEngine *engine,
 {
 	HTMLPainter *printer;
 	HTMLPainter *old_painter;
-	guint old_width, max_width;
 	GtkHTMLClassProperties *prop = GTK_HTML_CLASS (GTK_OBJECT (engine->widget)->klass)->properties;
 
 	g_return_if_fail (engine->clue != NULL);
 
-	old_width   = engine->width;
 	old_painter = engine->painter;
 	printer     = html_printer_new (print_context);
 	html_font_manager_set_default (&printer->font_manager,
 				       prop->font_var_print,      prop->font_fix_print,
 				       prop->font_var_size_print, prop->font_fix_size_print);
 
-	max_width = engine->width = html_printer_get_page_width (HTML_PRINTER (printer));
-
 	gtk_object_ref (GTK_OBJECT (old_painter));
-	html_engine_set_painter (engine, printer, max_width);
+	html_engine_set_painter (engine, printer);
 
 	print_all_pages (HTML_PAINTER (printer), 
 			 engine, 
@@ -165,9 +161,7 @@ html_engine_print_with_header_footer (HTMLEngine *engine,
 			 footer_print, 
 			 user_data);
 
-	engine->width = old_width;
-
-	html_engine_set_painter (engine, old_painter, old_width);
+	html_engine_set_painter (engine, old_painter);
 	gtk_object_unref (GTK_OBJECT (old_painter));
 	gtk_object_unref (GTK_OBJECT (printer));	
 }
