@@ -49,6 +49,7 @@
 #include "htmlenumutils.h"
 #include "htmlimage.h"
 #include "htmlobject.h"
+#include "htmlmap.h"
 #include "htmlprinter.h"
 
 /* HTMLImageFactory stuff.  */
@@ -61,7 +62,7 @@ struct _HTMLImageFactory {
 
 #define DEFAULT_SIZE 48
 #define ANIMATIONS(i) GTK_HTML_CLASS (GTK_OBJECT (i->image_ptr->factory->engine->widget)->klass)->properties->animations
-
+#define STRDUP_HELPER(i,j) if (i != j) {char *tmp = g_strdup (j); g_free(i); i = tmp;}
 
 HTMLImageClass html_image_class;
 static HTMLObjectClass *parent_class = NULL;
@@ -165,9 +166,11 @@ destroy (HTMLObject *o)
 	if (image->animation)
 		html_image_animation_destroy (image->animation);
 
-	if (image->url)    g_free (image->url);
-	if (image->target) g_free (image->target);
-	if (image->alt)    g_free (image->alt);
+	g_free (image->url);
+	g_free (image->target);
+	g_free (image->alt);
+	g_free (image->usemap);
+	g_free (image->final_url);
 
 	if (image->color)
 		html_color_unref (image->color);
@@ -200,6 +203,7 @@ copy (HTMLObject *self,
 	dimg->specified_height = simg->specified_height;
 	dimg->percent_width = simg->percent_width;
 	dimg->percent_height = simg->percent_height;
+	dimg->ismap = simg->ismap;
 
 	dimg->hspace = simg->hspace;
 	dimg->vspace = simg->vspace;
@@ -209,11 +213,68 @@ copy (HTMLObject *self,
 	dimg->url = g_strdup (simg->url);
 	dimg->target = g_strdup (simg->target);
 	dimg->alt = g_strdup (simg->alt);
+	dimg->usemap = g_strdup (simg->usemap);
+	dimg->final_url = NULL;
 
 	/* add dest to image_ptr interests */
 	dimg->image_ptr->interests = g_slist_prepend (dimg->image_ptr->interests, dimg);
 }
 
+static void 
+image_update_url (HTMLImage *image, gint x, gint y)
+{
+	HTMLMap *map;
+	HTMLObject *o = HTML_OBJECT (image);
+	char *url = NULL;
+
+	/* 
+	 * FIXME this is a huge hack waiting until we implement events for now we write
+	 * over the image->url for every point since we always call point before get_url
+	 * it is sick, I know.
+	 */
+	if (image->usemap != NULL) {
+		map = (HTMLMap *)html_engine_get_object_by_id (image->image_ptr->factory->engine, 
+							       image->usemap + 1);
+		
+		if (map) {
+			url = html_map_calc_point (map, x - o->x , y - (o->y - o->ascent));
+			
+			if (url)
+				url = g_strdup (url);
+		}
+	} else if (image->ismap) {
+		if (image->url)
+			url = g_strdup_printf ("%s?%d,%d", image->url, x - o->x, y - (o->y - o->ascent));
+	} else {
+		if (image->url)
+			url = g_strdup (image->url);
+	}
+	
+	g_free (image->final_url);
+	image->final_url = url;
+}
+
+static HTMLObject *
+check_point (HTMLObject *self,
+	     HTMLPainter *painter,
+	     gint x, gint y,
+	     guint *offset_return,
+	     gboolean for_cursor)
+{
+	if ((x >= self->x)
+	    && (x < (self->x + self->width))
+	    && (y >= (self->y - self->ascent))
+	    && (y < (self->y + self->descent))) {
+		if (offset_return != NULL)
+			*offset_return = 0;
+		
+		image_update_url (HTML_IMAGE (self), x, y);
+		return self;
+	}
+	
+	return NULL;
+}
+
 static gint
 calc_min_width (HTMLObject *o,
 		HTMLPainter *painter)
@@ -495,7 +556,7 @@ get_url (HTMLObject *o)
 	HTMLImage *image;
 
 	image = HTML_IMAGE (o);
-	return image->url;
+	return image->final_url;
 }
 
 static const gchar *
@@ -507,26 +568,26 @@ get_target (HTMLObject *o)
 	return image->target;
 }
 
+static const gchar *
+get_src (HTMLObject *o)
+{
+	HTMLImage *image;
+	
+	image = HTML_IMAGE (o);
+	return image->image_ptr->url;
+}
+
 static HTMLObject *
 set_link (HTMLObject *self, HTMLColor *color, const gchar *url, const gchar *target)
 {
 	HTMLImage *image = HTML_IMAGE (self);
 
-	if (image->url)    g_free (image->url);
-	if (image->target) g_free (image->target);
-
-	image->url = g_strdup (url);
-	image->target = g_strdup (target);
+	STRDUP_HELPER (image->url, url);
+	STRDUP_HELPER (image->target, target);
 
 	return NULL;
 }
 
-static HTMLObject *
-remove_link (HTMLObject *self, HTMLColor *color)
-{
-	return set_link (self, NULL, NULL, NULL);
-}
-
 static gboolean
 accepts_cursor (HTMLObject *o)
 {
@@ -581,10 +642,11 @@ html_image_class_init (HTMLImageClass *image_class,
 	object_class->calc_min_width = calc_min_width;
 	object_class->calc_preferred_width = calc_preferred_width;
 	object_class->calc_size = calc_size;
+	object_class->check_point = check_point;
 	object_class->get_url = get_url;
 	object_class->get_target = get_target;
+	object_class->get_src = get_src;
 	object_class->set_link = set_link;
-	object_class->remove_link = remove_link;
 	object_class->accepts_cursor = accepts_cursor;
 	object_class->get_valign = get_valign;
 	object_class->save = save;
@@ -616,7 +678,10 @@ html_image_init (HTMLImage *image,
 	html_object_init (object, HTML_OBJECT_CLASS (klass));
 
 	image->url = g_strdup (url);
-	image->target = g_strdup (url);
+	image->target = g_strdup (target);
+	image->usemap = NULL;
+	image->final_url = NULL;
+	image->ismap = FALSE;
 
 	image->specified_width  = width;
 	image->specified_height = height;
@@ -726,12 +791,26 @@ html_image_set_border (HTMLImage *image, gint border)
 void
 html_image_set_alt (HTMLImage *image, gchar *alt)
 {
-	if (image->alt)
-		g_free (image->alt);
-
+	g_free (image->alt);
 	image->alt = g_strdup (alt);
 }
 
+void
+html_image_set_map (HTMLImage *image, gchar *usemap, gboolean ismap)
+{
+	char *url = NULL;
+
+	g_free (image->usemap);
+
+	if (usemap != NULL) {
+		image->ismap = FALSE;
+		url = g_strdup (usemap);
+	} else {
+		image->ismap = ismap;
+	}
+	image->usemap = url;
+}
+
 void
 html_image_set_size (HTMLImage *image, gint w, gint h, gboolean pw, gboolean ph)
 {
@@ -761,7 +840,19 @@ html_image_set_size (HTMLImage *image, gint w, gint h, gboolean pw, gboolean ph)
 		html_engine_schedule_update (image->image_ptr->factory->engine);
 }
 
+char *image_content_types[] = {"image/*", NULL};
 
+static char **
+html_image_factory_types (GtkHTMLStream *stream,
+			  gpointer user_data)
+{
+	/* FIXME: this should use the not currently existant function
+	 * in gdk-pixbuf that tells us what mime types are supported 
+	 * by the modules it has loaded.
+	 */
+	return image_content_types;
+}
+
 static void
 html_image_factory_end_pixbuf (GtkHTMLStream *stream,
 			       GtkHTMLStreamStatus status,
@@ -788,6 +879,12 @@ html_image_factory_write_pixbuf (GtkHTMLStream *stream,
 	gdk_pixbuf_loader_write (p->loader, buffer, size);
 }
 
+static void
+html_image_factory_area_updated (GdkPixbufLoader *loader, guint x, guint y, guint width, guint height)
+{
+
+}
+
 static void
 html_image_factory_area_prepared (GdkPixbufLoader *loader, HTMLImagePointer *ip)
 {
@@ -801,7 +898,7 @@ html_image_factory_area_prepared (GdkPixbufLoader *loader, HTMLImagePointer *ip)
 		cur = ip->interests;
 		while (cur) {
 			if (cur->data)
-				html_object_change_set (HTML_OBJECT (cur->data), HTML_CHANGE_ALL);
+				html_object_change_set (HTML_OBJECT (cur->data), HTML_CHANGE_ALL_CALC);
 			cur = cur->next;
 		}
 
@@ -953,7 +1050,7 @@ html_image_factory_frame_done (GdkPixbufLoader *loader, GdkPixbufFrame *frame, H
 static void
 html_image_factory_animation_done (GdkPixbufLoader *loader, HTMLImagePointer *ip)
 {
-	printf ("animation done\n");
+	g_print ("animation done\n");
 }
 
 HTMLImageFactory *
@@ -1115,6 +1212,23 @@ html_image_pointer_unref (HTMLImagePointer *ip)
 	}
 }
 
+static void
+html_image_pointer_load (HTMLImagePointer *ip)
+{
+	GtkHTMLStream *handle;
+
+	html_image_pointer_ref (ip);
+	handle = gtk_html_stream_new (GTK_HTML (ip->factory->engine->widget),
+				      html_image_factory_types,
+				      html_image_factory_write_pixbuf,
+				      html_image_factory_end_pixbuf,
+				      ip);
+
+	/* This is a bit evil, I think.  But it's a lot better here
+	   than in the HTMLImage object.  FIXME anyway -- ettore  */
+	gtk_signal_emit_by_name (GTK_OBJECT (ip->factory->engine), "url_requested", ip->url, handle);
+}
+
 HTMLImagePointer *
 html_image_factory_register (HTMLImageFactory *factory, HTMLImage *i, const char *filename)
 {
@@ -1126,14 +1240,16 @@ html_image_factory_register (HTMLImageFactory *factory, HTMLImage *i, const char
 	retval = g_hash_table_lookup (factory->loaded_images, filename);
 
 	if (!retval){
-		GtkHTMLStream *handle;
-
 		retval = html_image_pointer_new (filename, factory);
 		if (*filename) {
 			gtk_signal_connect (GTK_OBJECT (retval->loader), "area_prepared",
 					    GTK_SIGNAL_FUNC (html_image_factory_area_prepared),
 					    retval);
 
+			gtk_signal_connect (GTK_OBJECT (retval->loader), "area_updated",
+					    GTK_SIGNAL_FUNC (html_image_factory_area_updated),
+					    retval);
+
 			gtk_signal_connect (GTK_OBJECT (retval->loader), "frame_done",
 					    GTK_SIGNAL_FUNC (html_image_factory_frame_done),
 					    retval);
@@ -1142,22 +1258,13 @@ html_image_factory_register (HTMLImageFactory *factory, HTMLImage *i, const char
 					    GTK_SIGNAL_FUNC (html_image_factory_animation_done),
 					    retval);
 
-			html_image_pointer_ref (retval);
-			handle = gtk_html_stream_new (GTK_HTML (factory->engine->widget),
-						      html_image_factory_write_pixbuf,
-						      html_image_factory_end_pixbuf,
-						      retval);
-
 			g_hash_table_insert (factory->loaded_images, retval->url, retval);
-
-		/* This is a bit evil, I think.  But it's a lot better here
-		   than in the HTMLImage object.  FIXME anyway -- ettore  */
-		
-			gtk_signal_emit_by_name (GTK_OBJECT (factory->engine), "url_requested", filename,
-						 handle);
+			html_image_pointer_load (retval);
 		}
 	}
 
+	html_image_pointer_ref (retval);
+
 	/* we add also NULL ptrs, as we dont want these to be cleaned out */
 	retval->interests = g_slist_prepend (retval->interests, i);
 
@@ -1173,10 +1280,23 @@ html_image_factory_register (HTMLImageFactory *factory, HTMLImage *i, const char
 	return retval;
 }
 
+#if 0
+HTMLEngine *
+html_image_factory_get_engine (HTMLImageFactory *factory)
+{
+	return factory->engine;
+}
+#endif
+
 void
 html_image_factory_unregister (HTMLImageFactory *factory, HTMLImagePointer *pointer, HTMLImage *i)
 {
 	pointer->interests = g_slist_remove (pointer->interests, i);
+	if (pointer->refcount <= 1) {
+		g_assert (pointer->interests);
+		g_hash_table_remove (factory->loaded_images, pointer->url);
+	}
+	html_image_pointer_unref (pointer);
 }
 
 static void
@@ -1203,6 +1323,25 @@ html_image_factory_stop_animations (HTMLImageFactory *factory)
 	g_hash_table_foreach (factory->loaded_images, stop_anim, NULL);
 }
 
+static gboolean
+move_image_pointers (gpointer key, gpointer value, gpointer data)
+{
+	HTMLImageFactory *dst = HTML_IMAGE_FACTORY (data);
+	HTMLImagePointer *ip  = HTML_IMAGE_POINTER (value);
+
+	ip->factory = dst;
+	g_hash_table_insert (dst->loaded_images, ip->url, ip);
+	html_image_pointer_load (ip);
+
+	return TRUE;
+}
+
+void
+html_image_factory_move_images (HTMLImageFactory *dst, HTMLImageFactory *src)
+{
+	g_hash_table_foreach_remove (src->loaded_images, move_image_pointers, dst);
+}
+
 static void
 deactivate_anim (gpointer key, gpointer value, gpointer user_data)
 {
