@@ -178,12 +178,13 @@ html_engine_select_line (HTMLEngine *e)
 gboolean
 html_engine_is_selection_active (HTMLEngine *e)
 {
+	html_engine_edit_selection_updater_do_idle (e->selection_updater);
 	return e->selection ? TRUE : FALSE;
 }
 
 void
 html_engine_activate_selection (HTMLEngine *e, guint32 time)
 {
-	if (e->selection)
+	if (e->selection && GTK_WIDGET_REALIZED (e->widget))
 		gtk_selection_owner_set (GTK_WIDGET (e->widget), GDK_SELECTION_PRIMARY, time);	
 }
